@@ -21,18 +21,20 @@ class Main(object):
     def __init__(self):
         # Parse command arguments.
         (log_level, updatedb, check_path, names, ignores, save_path,
-         project_path) = parse_args()
+         project_path, comparison_operator) = parse_args()
         # Enable logging.
         enable_pretty_logging(log_level=log_level)
         # Just allow do one thing at each time.
         if updatedb:
             self.update_db()
         elif check_path:
-            self.check_reqs_latest_version(check_path, ignores)
+            self.check_reqs_latest_version(
+                check_path, ignores, comparison_operator)
         elif names:
             self.search_package_by_name(names)
         else:
-            self.generate_reqs(save_path, project_path, ignores)
+            self.generate_reqs(save_path, project_path,
+                               ignores, comparison_operator)
 
     @property
     def installed_pkgs(self):
@@ -57,7 +59,8 @@ class Main(object):
             msg += 'Maybe you need update database.'
             print(Color.YELLOW(msg))
 
-    def check_reqs_latest_version(self, check_path, ignores):
+    def check_reqs_latest_version(self, check_path, ignores,
+                                  comparison_operator):
         """Check requirements latest version."""
         print(Color.BLUE('Starting check requirements latest version ...'))
         files = list()
@@ -74,7 +77,8 @@ class Main(object):
                 print(Color.YELLOW('Requirements file not found, '
                                    'generate requirements ...'))
                 save_path = os.path.join(check_path, 'requirements.txt')
-                self.generate_reqs(save_path, check_path, ignores)
+                self.generate_reqs(save_path, check_path,
+                                   ignores, comparison_operator)
                 files.append(save_path)
         else:
             files.append(check_path)
@@ -97,18 +101,23 @@ class Main(object):
         print()
         print_table(pkg_versions)
 
-    def generate_reqs(self, save_path, check_path, ignores):
-        gr = GenerateReqs(save_path, check_path, ignores, self.installed_pkgs)
+    def generate_reqs(self, save_path, check_path,
+                      ignores, comparison_operator):
+        gr = GenerateReqs(save_path, check_path, ignores,
+                          self.installed_pkgs, comparison_operator)
         gr.generate_reqs()
 
 
 class GenerateReqs(object):
 
-    def __init__(self, save_path, project_path, ignores, installed_pkgs):
+    def __init__(self, save_path, project_path, ignores,
+                 installed_pkgs, comparison_operator='=='):
         self._save_path = save_path
         self._project_path = project_path
         self._ignores = ignores
         self._installed_pkgs = installed_pkgs
+        self._maybe_local_mods = set()
+        self._comparison_operator = comparison_operator
 
     def generate_reqs(self):
         """Generate requirements for `project_path`, save file in
@@ -120,15 +129,13 @@ class GenerateReqs(object):
         answer = 'n'
         pyver = None
 
+        guess.remove(*try_imports)
         if guess:
-            pyver = 'Python 3' if sys.version_info[0] == 2 else 'Python 2'
             print(Color.RED('The following modules are not found yet:'))
-            for name, detail in guess.items():
-                print('  {0} referenced from:\n    {1}'.format(
-                    Color.YELLOW(name), '\n    '.join(detail.comments)))
-            msg = ('Some of them may come from other Python version '
-                   '(i.e {0}).\nTry to search PyPI for the missing '
-                   'modules and filter some unnecessary modules? [y/N] '
+            self._invalid_reqs(guess)
+            msg = ('Some of them may not install in local environment.\n'
+                   'Try to search PyPI for the missing modules and filter'
+                   ' some unnecessary modules? (y/[N]) '
                    ).format(pyver)
             sys.stdout.write(Color.RED(msg))
             sys.stdout.flush()
@@ -136,16 +143,18 @@ class GenerateReqs(object):
             answer = answer.strip().lower()
             if answer in ('y', 'yes'):
                 print(Color.BLUE('Checking modules on the PyPI...'))
-                for name in guess:
-                    logger.info('Checking {0} on the PyPI ...'.format(name))
+                for name, detail in guess.sorted_items():
+                    logger.info('Checking %s on the PyPI ...', name)
                     with database() as db:
                         rows = db.query_all(name)
                         pkgs = [row.package for row in rows]
                         if pkgs:
                             in_pypi.add(name)
+                            if name in self._maybe_local_mods:
+                                self._maybe_local_mods.remove(name)
                         for pkg in self._best_matchs(name, pkgs):
                             latest = check_latest_version(pkg)
-                            reqs.add(pkg, latest, guess[name].comments)
+                            reqs.add(pkg, latest, detail.comments)
 
         # Save old requirements file.
         self._save_old_reqs()
@@ -157,12 +166,10 @@ class GenerateReqs(object):
         del reqs
 
         if guess and answer in ('y', 'yes'):
-            guess.remove(*(in_pypi | try_imports))
+            guess.remove(*(in_pypi | self._maybe_local_mods))
             if guess:
                 print(Color.RED('These modules are not found:'))
-                for name, detail in guess.items():
-                    print('  {0} referenced from:\n    {1}'.format(
-                        Color.YELLOW(name), '\n    '.join(detail.comments)))
+                self._invalid_reqs(guess)
                 print(Color.RED('Maybe or you need update database.'))
 
     def extract_reqs(self):
@@ -171,13 +178,16 @@ class GenerateReqs(object):
         guess = ReqsModules()
         modules, try_imports, local_mods = project_import_modules(
             self._project_path, self._ignores)
+        app_name = os.path.basename(self._project_path)
+        if app_name in local_mods:
+            local_mods.remove(app_name)
 
         # Filtering modules
         candidates = self._filter_modules(modules, local_mods)
 
         logger.info('Check module in local environment.')
         for name in candidates:
-            logger.info('Checking module: {0}'.format(name))
+            logger.info('Checking module: %s', name)
             if name in self._installed_pkgs:
                 pkg_name, version = self._installed_pkgs[name]
                 reqs.add(pkg_name, version, modules[name])
@@ -191,14 +201,16 @@ class GenerateReqs(object):
             self._save_path)))
         with open(self._save_path, 'w+') as f:
             f.write('# Requirements automatically generated by pigar.\n'
-                    '# https://github.com/Damnever/pigar\n')
-            for k, v in reqs.items():
+                    '# https://github.com/damnever/pigar\n')
+            for k, v in reqs.sorted_items():
                 f.write('\n')
-                f.write(''.join(['# {0}\n'.format(c) for c in v.comments]))
+                f.write(''.join(['# {0}\n'.format(c)
+                                 for c in v.comments.sorted_items()]))
                 if k == '-e':
                     f.write('{0} {1}\n'.format(k, v.version))
                 elif v:
-                    f.write('{0} == {1}\n'.format(k, v.version))
+                    f.write('{0} {1} {2}\n'.format(
+                        k, self._comparison_operator, v.version))
                 else:
                     f.write('{0}\n'.format(k))
 
@@ -214,17 +226,26 @@ class GenerateReqs(object):
 
         logger.info('Filtering modules ...')
         for module in modules:
-            logger.info('Checking module: {0}'.format(module))
+            logger.info('Checking module: %s', module)
             if not module or module.startswith('.'):
                 continue
             if module in local_mods:
-                continue
+                self._maybe_local_mods.add(module)
             if is_stdlib(module):
                 continue
             candidates.add(module)
 
         return candidates
 
+    def _invalid_reqs(self, reqs):
+        for name, detail in reqs.sorted_items():
+            print(
+                '  {0} referenced from:\n    {1}'.format(
+                    Color.YELLOW(name),
+                    '\n    '.join(detail.comments.sorted_items())
+                )
+            )
+
     def _save_old_reqs(self):
         if os.path.isfile(self._save_path):
             with codecs.open(self._save_path, 'rb', 'utf-8') as f:
@@ -248,5 +269,6 @@ class GenerateReqs(object):
 def main():
     Main()
 
+
 if __name__ == '__main__':
     main()
